# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import, with_statement

from wtforms import Form, TextField, TextAreaField, validators
from flask import request, session, render_template, redirect, abort, g, \
                  url_for, flash

from . import app, login_required, admin_required
from ..model import User, Realm
from .. import migrations


class PossessionForm(Form):
    nick = TextField('Nick', [validators.Required()])

    def validate_nick(form, field):
        if not User.find_by_nick(field.data):
            raise validators.ValidationError("unknown Nickname")


@app.route("/admin/")
@admin_required
def admin_index():
    return render_template("admin/index.html.jinja")


@app.route("/admin/migrations/", methods=["GET", "POST"])
def migrate():
    pending_migrations = migrations.get_pending_migrations()
    if request.method == "POST":
        for name, desc, func in pending_migrations:
            func()
        return redirect(url_for("migrate"))
    else:
        return render_template("admin/migrations.html.jinja",
                                         pending_migrations=pending_migrations)


@app.route("/admin/possess/", methods=["GET", "POST"])
@admin_required
def possess():
    if request.method == "POST":
        form = PossessionForm(request.form)
        if form.validate():
            user = User.find_by_nick(form.data['nick'])
            session['possessing_openid'] = g.user.openid
            session['openid'] = user.openid
            return redirect(url_for('show_profile'))
    else:
        form = PossessionForm()

    return render_template("admin/possess.html.jinja", form=form)
