# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import
from functools import wraps
from flask import g


_migrations = []


def migrationhandler(func):
    @wraps(func)
    def wrapper():
        doc = get_migration_doc()
        doc["migrations"].append(func.__name__)
        func()
        g.db.admin.save(doc)

    desc = func.__doc__ and func.__doc__.strip() or ''

    _migrations.append((func.__name__, desc, wrapper))
    return wrapper


def get_migration_doc():
    empty = {
        '_id': 'migrations',
        'migrations': [],
    }
    return g.db.admin.find_one({'_id': 'migrations'}) or empty


def get_pending_migrations():
    doc = get_migration_doc()
    return [(name, desc, func) for name, desc, func in _migrations
                               if name not in doc["migrations"]]


@migrationhandler
def add_index_user_nick():
    """ Adds an index to the users collection, so that `nick` is unique """
    g.db.users.ensure_index("nick")


@migrationhandler
def migration_add_notifications_to_users():
    """ Adds the field 'notifications': [] to every user.
    """
    g.db.users.update({}, {'$set': {'notifications': []}}, multi=True)


@migrationhandler
def update_realms_for_multiple_owners():
    """ Updates all Realms, so that instead of a owner they have a list of
        owners
    """
    for doc in g.db.realms.find():
        doc["owners"] = [doc.pop("owner")]
        g.db.realms.save(doc)


@migrationhandler
def add_admin_field_to_user():
    """Adds the field 'is_admin': False to every user. """
    g.db.users.update({}, {'$set': {'is_admin': False}}, multi=True)
