# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import

from flask import g


def docproperty(name):
    def get_value(self):
        return self.doc[name]

    def set_value(self, value):
        self.doc[name] = value
    return property(get_value, set_value)


def doclistproperty(name, factory):

    def get_value(self):
        value = self.doc
        for namepart in name.split('.'):
            value = value[namepart]
        return [factory(subdoc, parent=self) for subdoc in value]

    return property(get_value)


class Persistent(object):

    def __init__(self, doc=None, parent=None):
        self.parent = parent
        if doc is None:
            self.doc = self.empty_doc()
        else:
            self.doc = doc

    def __repr__(self):
        return '<%s %s>' % (type(self).__name__, self.doc)


class Document(Persistent):
    """ Base class for all top-level documents, that reside in collections
    """

    @classmethod
    def collection(cls):
        """ The collection, in which instances of this class are stored.

        requires `collection_name` to be set.
        """
        return g.db[cls.collection_name]

    @classmethod
    def find_one_by_filter(cls, filter):
        """ Find the first Document in the database matching the filter

        filter
            a mongodb filter, e.g. {'attribute': 'value'}
        return
            the found object, or None
        """
        doc = cls.collection().find_one(filter)
        return doc and cls(doc)

    @classmethod
    def find_by_filter(cls, filter):
        """ Find all documents in the database matching the filter

        filter
            a mongodb filter, e.g. {'attribute': 'value'}
        return
            a (possibly empty) list of found objects
        """
        return [cls(doc) for doc in cls.collection().find(filter)]

    def save(self):
        """ Inserts or updates the document in the database
        """
        self.collection().save(self.doc)

    def remove(self):
        """ Removes the document from the database
        """
        self.collection().remove(self.doc['_id'])
