# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import

import bson
from datetime import datetime

from .base import Persistent, docproperty


class Notification(Persistent):
    """ Document Embedded in User.notifications[]

    Example documents

    {
        'id': ObjectId('C0FFEEBABE'),
        'text': 'you gained the achievement!',
        'date': '2011-18-11 23:05:17',
    }
    """
    id = docproperty("id")
    text = docproperty("text")
    date = docproperty("date")

    def __init__(self, mother, subdoc):
        self.mother = mother
        self.doc = subdoc

    def remove(self):
        self.mother.doc["notifications"].remove(self.doc)

    @classmethod
    def create(cls, user, text, date=None):
        doc = {
            'id': bson.ObjectId(),
            'text': text,
            'date': date or datetime.now(),
        }

        user.doc['notifications'].append(doc)
        return cls(user, doc)
